﻿using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;

namespace MVRPlugin {
	public class SliderAutoUpdate : MVRScript
    {
        SliderControl targetSliderControl;
        UISliderTrigger targetTrigger;

        Atom targetAtom;
        JSONStorable targetReceiver;
        JSONStorableFloat targetFloatValue;
        float targetFloatMax, targetFloatMin;
		
		public override void Init()
        {
			try
            {
                Atom parent = null;

                if(containingAtom.parentAtom != null)
                {
                    parent = containingAtom.parentAtom;
                    containingAtom.parentAtom = null;
                }

                targetSliderControl = GetContainingAtom().GetComponentInChildren<SliderControl>();
                targetTrigger = GetContainingAtom().GetComponentInChildren<UISliderTrigger>();

                RefreshTarget();

                if(parent != null)
                {
                    containingAtom.parentAtom = parent;
                }
            }
            catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		// Start is called once before Update or FixedUpdate is called and after Init()
		void Start() {
			try {
                UIDynamicButton button = CreateButton("Refresh", false);
                button.button.onClick.AddListener(RefreshTarget);
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		// Update is called with each rendered frame by Unity
		void Update() {
			try
            {
                if (targetFloatValue == null)
                    return;

                targetSliderControl.setSliderToValue(map(targetFloatValue.val, targetFloatMin, targetFloatMax, 0, 1));
            }
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		// FixedUpdate is called with each physics simulation frame by Unity
		void FixedUpdate() {
			try {
				// put code in here
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

        void RefreshTarget()
        {
            foreach (Atom a in SuperController.singleton.GetAtoms())
            {              
                if (a.name == targetTrigger.trigger.GetJSON()["transitionActions"][0]["receiverAtom"].Value)
                {
                    targetAtom = a;
                    break;
                }
            }

            if (targetAtom == null)
                return;

            targetReceiver = targetAtom.GetStorableByID(targetTrigger.trigger.GetJSON()["transitionActions"][0]["receiver"].Value);

            if (targetReceiver == null)
                return;
               
            targetFloatValue = targetReceiver.GetFloatJSONParam(targetTrigger.trigger.GetJSON()["transitionActions"][0]["receiverTargetName"].Value);

            float.TryParse(targetTrigger.trigger.GetJSON()["transitionActions"][0]["startValue"].Value, out targetFloatMin);
            float.TryParse(targetTrigger.trigger.GetJSON()["transitionActions"][0]["endValue"].Value, out targetFloatMax);
        }

        float map(float s, float a1, float a2, float b1, float b2)
        {
            return b1 + (s - a1) * (b2 - b1) / (a2 - a1);
        }

        // OnDestroy is where you should put any cleanup
        // if you registered objects to supercontroller or atom, you should unregister them here
        void OnDestroy() {
		}

	}
}